local htmlEntities = module("lib/htmlEntities")

local cfg = module("cfg/identity")
local lang = vRP.lang

local sanitizes = module("cfg/sanitizes")

function vRP.getUserIdentity(user_id, cbr)
  local task = Task(cbr)
  exports.oxmysql:execute("SELECT * FROM vrp_users WHERE id = @user_id", {["@user_id"] = user_id}, function(rows)
    task({rows[1]})
  end)
end

-- cbreturn user_id by registration or nil
function vRP.getUserByRegistration(registration, cbr)
  local task = Task(cbr)
  exports.oxmysql:execute("SELECT id FROM vrp_users WHERE registration = @registration", {["@registration"] = registration or ""}, function(rows)
    if #rows > 0 then
      task({rows[1].user_id})
    else
      task()
    end
  end)
end

AddEventHandler("vRP:playerSpawn", function(user_id, source, first_spawn)
  exports["oxmysql"]:query("SELECT buletin FROM vrp_users WHERE id = @user_id", {user_id = user_id}, function(result)
    buletin = result[1].buletin
    if buletin then
    else
      TriggerClientEvent("R01:identity:createCharacter", source)
    end
  end)
end)

-- cbreturn user_id by phone or nil
function vRP.getUserByPhone(phone, cbr)
  local task = Task(cbr)
  exports.oxmysql:execute("SELECT id FROM vrp_users WHERE phone = @phone", {["@phone"] = phone or ""}, function(rows)
    if #rows > 0 then
      task({rows[1].user_id})
    else
      task()
    end
  end)
end

function vRP.generateStringNumber(format) -- (ex: DDDLLL, D => digit, L => letter)
  local abyte = string.byte("A")
  local zbyte = string.byte("0")

  local number = ""
  for i=1,#format do
    local char = string.sub(format, i,i)
    if char == "D" then number = number..string.char(zbyte+math.random(0,9))
    elseif char == "L" then number = number..string.char(abyte+math.random(0,25))
    else number = number..char end
  end

  return number
end

-- cbreturn a unique registration number
function vRP.generateRegistrationNumber(cbr)
  local task = Task(cbr)

  local function search()
    -- generate registration number
    local registration = vRP.generateStringNumber("DDDLLL")
    vRP.getUserByRegistration(registration, function(user_id)
      if user_id ~= nil then
        search() -- continue generation
      else
        task({registration})
      end
    end)
  end

  search()
end

-- cbreturn a unique phone number (0DDDDD, D => digit)
function vRP.generatePhoneNumber(cbr)
  local task = Task(cbr)

  local function search()
    -- generate phone number
    local phone = vRP.generateStringNumber(cfg.phone_format)
    vRP.getUserByPhone(phone, function(user_id)
      if user_id ~= nil then
        search() -- continue generation
      else
        task({phone})
      end
    end)
  end

  search()
end

-- events, init user identity at connection
AddEventHandler("vRP:playerJoin",function(user_id,source,name,last_login)
  vRP.getUserIdentity(user_id, function(identity)
      vRP.generateRegistrationNumber(function(registration)
          exports.oxmysql:execute("INSERT IGNORE INTO vrp_users(id,registration,phone,firstname,name,age) VALUES(@user_id,@registration,@phone,@firstname,@name,@age)", {
            ["@user_id"] = user_id,
            ["@registration"] = registration,
            ["@phone"] = phone,
            ["@firstname"] = cfg.random_first_names[math.random(1,#cfg.random_first_names)],
            ["@name"] = cfg.random_last_names[math.random(1,#cfg.random_last_names)],
            ["@age"] = 13
          })
      end)
  end)
end)

-- city hall menu

local cityhall_menu = {name=lang.cityhall.title(),css={top="75px", header_color="rgba(0,125,255,0.75)"}}

function checkName(theText)
	local foundSpace, valid = false, true
	local spaceBefore = false
	local current = ''
	for i = 1, #theText do
		local char = theText:sub( i, i )
		if char == ' ' then 
			if i == #theText or i == 1 or spaceBefore then 
				valid = false
				break
			end
			current = ''
			spaceBefore = true
		elseif ( char >= 'a' and char <= 'z' ) or ( char >= 'A' and char <= 'Z' ) then 
			current = current .. char
			spaceBefore = false
		else 
			valid = false
			break
		end
	end
	
	if (valid == true)  then
		return true
	else
		return false
	end
end

local function ch_identity(player,choice)
  local user_id = vRP.getUserId(player)
  local name1 = ""
  local name2 = ""
  if user_id ~= nil then
    vRP.prompt(player,lang.cityhall.identity.prompt_firstname(),"",function(player,firstname)
      if string.len(firstname) >= 2 and string.len(firstname) < 50 then
    local name1 = firstname
        firstname = sanitizeString(firstname, sanitizes.name[1], sanitizes.name[2])
        vRP.prompt(player,lang.cityhall.identity.prompt_name(),"",function(player,name)
          if string.len(name) >= 2 and string.len(name) < 50 then
      local name2 = name
            name = sanitizeString(name, sanitizes.name[1], sanitizes.name[2])
            vRP.prompt(player,lang.cityhall.identity.prompt_age(),"",function(player,age)
              age = parseInt(age)
        if age >= 16 and age <= 150 then
          if (checkName(name1)) then
            if (checkName(name2)) then
              if vRP.tryPayment(user_id,cfg.new_identity_cost) then
                exports.oxmysql:execute("UPDATE vrp_users SET firstname = @firstname, name = @name, age = @age WHERE id = @user_id", {
                  ["@user_id"] = user_id,
                  ["@firstname"] = firstname,
                  ["@name"] = name,
                  ["@age"] = age
                })

                -- update client registration
                --vRPclient.setRegistrationNumber(player,{registration})
                theAddress = "Bucuresti"
                vRP.getUserAddress(user_id, function(address)
                  if address ~= nil then
                    house = address.home
                    number = address.number
                    theAddress = address.home.." ["..address.number.."]"
                  else
                    theAddress = "Bucuresti"
                  end
                  idDesc = "<font color ='white'>C.N.P:<font> <font color='green'>"..user_id.."</font><br><font color ='white'>Nume:</font> <font color='green'>"..firstname.."</font><br><font color ='white'>Prenume:</font> <font color='green'>"..name.."</font><br><font color ='white'>Varsta:</font> <font color='green'>"..age.."</font><br><font color ='white'>Domiciliu:</font> <font color='green'>"..theAddress.."</font>"
                  if(vRP.tryGetInventoryItem(user_id,"id_doc",1,false))then
                    vRP.giveInventoryItem(user_id,"id_doc",1,true,idDesc)
                  else
                    vRP.giveInventoryItem(user_id,"id_doc",1,true,idDesc)
                  end
                end)
                vRPclient.notify(player,{lang.money.paid({cfg.new_identity_cost})})
              else
                vRPclient.notify(player,{lang.money.not_enough()})
              end
            else
              vRPclient.notify(player,{lang.common.invalid_value()})
            end
          else
            vRPclient.notify(player,{lang.common.invalid_value()})
          end
        else
          vRPclient.notify(player,{lang.common.invalid_value()})
        end
            end)
          else
            vRPclient.notify(player,{lang.common.invalid_value()})
          end
        end)
      else
        vRPclient.notify(player,{lang.common.invalid_value()})
      end
    end)
  end
end
cityhall_menu[lang.cityhall.identity.title()] = {ch_identity,lang.cityhall.identity.description({cfg.new_identity_cost})}

local function cityhall_enter()
  local user_id = vRP.getUserId(source)
  if user_id ~= nil then
    vRP.openMenu(source,cityhall_menu)
  end
end

local function cityhall_leave()
  vRP.closeMenu(source)
end

local function build_client_cityhall(source) -- build the city hall area/marker/blip
  local user_id = vRP.getUserId(source)
  if user_id ~= nil then
    local x,y,z = table.unpack(cfg.city_hall)

    --vRPclient.addBlip(source,{x,y,z,cfg.blip[1],cfg.blip[2],lang.cityhall.title()})
    vRPclient.addMarkerNames(source,{x,y,z, "Identitate", 0, 1.0})
    vRPclient.addMarker(source,{x,y,z-1,0.7,0.7,0.5,0,255,125,125,150})

    vRP.setArea(source,"Identitate",x,y,z,1,1.5,cityhall_enter,cityhall_leave)
  end
end

AddEventHandler("vRP:playerSpawn",function(user_id, source, first_spawn)
  -- send registration number to client at spawn
  vRP.getUserIdentity(user_id, function(identity)
    exports.oxmysql:execute("SELECT * FROM vrp_users WHERE id = @user_id", {["@user_id"] = user_id}, function(rows)
    end)
    if identity then
      vRPclient.setRegistrationNumber(source,{identity.registration or "000AAA"})
    end
  end)

  -- first spawn, build city hall
  if first_spawn then
    build_client_cityhall(source)
  end
end)
